﻿using System.Collections.Generic;
using System.Linq;

namespace CCars
{
    class AktuálisÚtKalkulátor : ISzenzorAdatFeldolgozó
    {
        /// <summary>
        /// A pillanatnyi idő
        /// </summary>
        public float Idő { get; set; }

        /// <summary>
        /// A szimuláció végének időpontja
        /// </summary>
        public float MaximálisIdő { get; private set; }

        /// <summary>
        /// A pillanatnyi pozíció
        /// </summary>
        public Vektor Pozíció
        {
            get
            {
                int i = mérések.FindIndex(m => m.Idő >= Idő);
                if (i == -1)
                {
                    return mérések.Last().Pozíció;
                }
                else if (i == 0)
                {
                    return mérések[0].Pozíció;
                }
                //Kvadratikus interpoláció
                float t = Idő - mérések[i].Idő;
                float a = mérések[i].Idő;
                float v0 = mérések[i - 1].Sebesség;
                float s = v0 * t + a / 2 * t * t;
                float táv = Vektor.Távolság(mérések[i - 1].Pozíció, mérések[i].Pozíció);
                return mérések[i - 1].Pozíció + (mérések[i].Pozíció - mérések[i - 1].Pozíció) * s / táv;
            }
        }

        /// <summary>
        /// A pillanatnyi sebesség
        /// </summary>
        public float Sebesség
        {
            get
            {
                int i = mérések.FindIndex(m => m.Idő >= Idő);
                if (i == -1 || i == 0)
                {
                    return 0;
                }
                //Lineáris interpoláció
                float t = Idő - mérések[i].Idő;
                float a = mérések[i].Idő;
                float v0 = mérések[i - 1].Sebesség;
                return v0 + a * t;
            }
        }

        /// <summary>
        /// A pillanatnyi gyorsulás
        /// </summary>
        public float Gyorsulás
        {
            get
            {
                int i = mérések.FindIndex(m => m.Idő >= Idő);
                if (i == -1)
                {
                    return 0;
                }
                return mérések[i].Gyorsulás;
            }
        }

        public bool MotorBeindítva
        {
            get
            {
                int i = motorEsemények.FindLastIndex(m => m.Idő < Idő);
                if(i == -1)
                {
                    return false;
                }
                return motorEsemények[i].Beindult;
            }
        }

        /// <summary>
        /// Adatok egy méréskor
        /// </summary>
        private struct Mérés
        {
            public float Idő;
            public float Sebesség;
            public float Gyorsulás;
            public Vektor Pozíció;
        }

        private List<Mérés> mérések = new List<Mérés>();

        private struct MotorEsemény
        {
            public float Idő;
            public bool Beindult;
        }

        private List<MotorEsemény> motorEsemények = new List<MotorEsemény>();

        void ISzenzorAdatFeldolgozó.MotorLeáll(float idő)
        {
            MotorEsemény es;
            es.Idő = idő;
            es.Beindult = false;
            motorEsemények.Add(es);
            MaximálisIdő = idő;
        }

        void ISzenzorAdatFeldolgozó.MotorBeindul(float idő)
        {
            MotorEsemény es;
            es.Idő = idő;
            es.Beindult = true;
            motorEsemények.Add(es);
            MaximálisIdő = idő;
        }

        void ISzenzorAdatFeldolgozó.Koordináta(float idő, Vektor v)
        {
            Mérés m;
            m.Idő = idő;
            m.Pozíció = v;
            if (mérések.Count == 0)
            {
                //Ez az első
                m.Sebesség = 0;
                m.Gyorsulás = 0;
            }
            else
            {
                //Kiszámoljuk az adatokat
                Mérés előző = mérések.Last();
                float v0 = előző.Sebesség;
                float t = idő - előző.Idő;
                float s = Vektor.Távolság(v, előző.Pozíció);
                float a = (s / t - v0 * t) * 2 / t;
                m.Sebesség = v0 + a * t;
                m.Gyorsulás = a;
            }
            mérések.Add(m);
            MaximálisIdő = idő;
        }
    }
}
