﻿using System;
using System.Globalization;
using System.Windows.Forms;

namespace CCars
{
    partial class Főablak : Form
    {
        public Főablak(AktuálisÚtKalkulátor aktuálisKalkulátor, TeljesÚtKalkulátor teljesKalkulátor, Térkép térkép)
        {
            InitializeComponent();
            AktuálisKalkulátor = aktuálisKalkulátor;
            TeljesKalkulátor = teljesKalkulátor;
            térképMegjelenítő.Térkép = térkép;
        }

        private AktuálisÚtKalkulátor AktuálisKalkulátor;
        private TeljesÚtKalkulátor TeljesKalkulátor;

        // Ha el van indítva a lejátszás
        private bool Elindítva = false;

        // Az időzítő ennyit ad hozzá az időhöz
        private float LejátszásiSebesség = 1;

        private float Idő = 0;

        private void Főablak_Load(object sender, EventArgs e)
        {
            AktuálisAdatMegjelenít(0);

            TeljesÚtMegjelenítő();

            // Happy end :)
        }

        /// <summary>
        /// Teljes út adatainak megjelenítése.
        /// </summary>
        private void TeljesÚtMegjelenítő()
        {
            TeljesÚtCímke.Text = TeljesKalkulátor.MegtettÚt.ToString();
            KiindulásCímke.Text = TeljesKalkulátor.KiindulásiPont.ToString();
            ÉrkezésCímke.Text = TeljesKalkulátor.ÉrkezésiPont.ToString();
            MaxSebességCímke.Text = TeljesKalkulátor.MaximálisSebesség.ToString();
            ÁtlagsebességCímke.Text = TeljesKalkulátor.ÁtlagSebesség.ToString();
            IncidensCímke.Text = TeljesKalkulátor.IncidensekSzáma.ToString();
        }

        /// <summary>
        /// Aktuális út adatainak megjelenítése.
        /// </summary>
        /// <param name="idő">Ehhez az időhöz</param>
        private void AktuálisAdatMegjelenít(float idő)
        {
            AktuálisKalkulátor.Idő = idő;

            if (AktuálisKalkulátor.MotorBeindítva == true)
            {
                MotorállapotCímke.Text = "BEINDÍTVA";
            }
            else
            {
                MotorállapotCímke.Text = "ÁLL";
            }

            AktuálisPozícióCímke.Text = AktuálisKalkulátor.Pozíció.ToString();

            AktuálisSebességCímke.Text = AktuálisKalkulátor.Sebesség.ToString();

            AktuálisGyorsulásCímke.Text = AktuálisKalkulátor.Gyorsulás.ToString();
            if (AktuálisKalkulátor.Gyorsulás >= 2.5)
            {
                HirtelenGyorsítCímke.Text = "IGEN";
            }
            else
            {
                HirtelenGyorsítCímke.Text = "NEM";
            }
            if (AktuálisKalkulátor.Gyorsulás <= -2.5)
            {
                HirtelenLassulCímke.Text = "IGEN";
            }
            else
            {
                HirtelenLassulCímke.Text = "NEM";
            }
            térképMegjelenítő.AutóPozíció = AktuálisKalkulátor.Pozíció;
        }

        private void startGomb_Click(object sender, EventArgs e)
        {
            Elindítva = !Elindítva;

            //időBeviteliMező.Enabled = !Elindítva;
            startGomb.Text = Elindítva ? "Stop" : "Start";
        }

        private void IdőtFrissít(float idő)
        {
            float csúszkaSzázalék = idő / (AktuálisKalkulátor.MaximálisIdő);
            if (csúszkaSzázalék >= 1)
            {
                if (Elindítva)
                {
                    Idő = 0;
                    Elindítva = false;
                    startGomb.Text = "Start";
                    időBeviteliMező.Text = "0";
                    időCsúszka.Value = 0;
                }
            }
            else
            {
                időCsúszka.Value = (int)(időCsúszka.Maximum * csúszkaSzázalék);

                időBeviteliMező.Text = idő + "";
            }
            AktuálisAdatMegjelenít(idő);
        }

        private void időzítő_Tick(object sender, EventArgs e)
        {
            if(Elindítva)
            {
                IdőtFrissít(Idő += LejátszásiSebesség);
            }
        }

        private void lejátszásiSebesség_Scroll(object sender, EventArgs e)
        {
            LejátszásiSebesség = lejátszásiSebesség.Value / 10;
        }

        private void időCsúszka_Scroll(object sender, EventArgs e)
        {
            Idő = (float)időCsúszka.Value / időCsúszka.Maximum * AktuálisKalkulátor.MaximálisIdő;
            IdőtFrissít(Idő);
        }

        private void időBeviteliMező_Enter(object sender, EventArgs e)
        {
            try
            {
                float újIdő = float.Parse(időBeviteliMező.Text, CultureInfo.InvariantCulture);
                IdőtFrissít(újIdő);
            }
            catch
            {

            }
        }
    }
}
