﻿using System.Collections.Generic;
using System.IO;
using System.Globalization;

namespace CCars
{
    /// <summary>
    /// Beolvassa a szenzor adatokat és feldolgoztatja a feldolgozókkal
    /// </summary>
    class SzenzorAdatBeolvasó
    {
        private List<ISzenzorAdatFeldolgozó> Feldolgozók;
        private string SzenzorAdatFájlnév;

        public SzenzorAdatBeolvasó(string szenzorAdatFájlnév, List<ISzenzorAdatFeldolgozó> feldolgozók)
        {
            SzenzorAdatFájlnév = szenzorAdatFájlnév;
            Feldolgozók = feldolgozók;
        }

        /// <summary>
        /// Szenzor adatok beolvasása és feldolgoztatása
        /// </summary>
        public void Beolvas()
        {
            using (StreamReader fájl = new StreamReader(SzenzorAdatFájlnév))
            {
                string sor;
                while ((sor = fájl.ReadLine()) != null)
                {
                    string[] adatrészek = sor.Split();
                    float idő = float.Parse(adatrészek[0], CultureInfo.InvariantCulture);
                    if (adatrészek[1] == "MOTOR-BE")
                    {
                        foreach (ISzenzorAdatFeldolgozó feldolgozó in Feldolgozók)
                            feldolgozó.MotorBeindul(idő);
                    }
                    else if (adatrészek[1] == "MOTOR-KI")
                    {
                        foreach (ISzenzorAdatFeldolgozó feldolgozó in Feldolgozók)
                            feldolgozó.MotorLeáll(idő);
                    }
                    else
                    {
                        Vektor hely = new Vektor();
                        hely.X = float.Parse(adatrészek[2], CultureInfo.InvariantCulture);
                        hely.Y = float.Parse(adatrészek[3], CultureInfo.InvariantCulture);
                        foreach (ISzenzorAdatFeldolgozó feldolgozó in Feldolgozók)
                            feldolgozó.Koordináta(idő, hely);
                    }
                }
            }
        }
    }
}
