﻿namespace CCars
{
    /// <summary>
    /// A teljes út adatai.
    /// </summary>
    class TeljesÚtKalkulátor : ISzenzorAdatFeldolgozó
    {
        private Térkép Térkép;

        public TeljesÚtKalkulátor(Térkép térkép)
        {
            Térkép = térkép;
        }

        public float MegtettÚt
        {
            get;
            private set;
        }

        public Vektor KiindulásiPont
        {
            get;
            private set;
        }

        public Vektor ÉrkezésiPont
        {
            get;
            private set;
        }

        public float ÁtlagSebesség
        {
            get
            {
                return MegtettÚt / (UtolsóLeállIdő - IndulásiIdő);
            }
        }

        public float MaximálisSebesség
        {
            get;
            private set;
        }

        public int IncidensekSzáma
        {
            get;
            private set;
        }

        void ISzenzorAdatFeldolgozó.MotorLeáll(float idő)
        {
            // átlag számításhoz az utolsó ismert idő
            UtolsóLeállIdő = idő;
        }

        private float IndulásiIdő = -1;
        void ISzenzorAdatFeldolgozó.MotorBeindul(float idő)
        {
            if (IndulásiIdő == -1)
            {
                IndulásiIdő = idő;
            }
        }

        private Vektor? ElőzőVektor = null;
        private float UtolsóLeállIdő = -1;
        private float UtolsóIdő = -1;
        private bool ElindultMár = false;
        void ISzenzorAdatFeldolgozó.Koordináta(float idő, Vektor v)
        {
            // Kiindulási pont "kiszámítása"
            if (!ElindultMár)
            {
                ElindultMár = true;
                KiindulásiPont = v;
            }

            /*
            * Megtett út kiszámítása
            */
            // Az aktuálisTávolság az előző koordináta óta megtett távolságot jelenti
            float aktuálisTávolság = (ElőzőVektor != null) ? Vektor.Távolság((Vektor)ElőzőVektor, v) : v.Hossz;
            MegtettÚt += aktuálisTávolság;

            /*
             * Max sebesség kiszámítása
             */
            float sebesség = aktuálisTávolság / (UtolsóIdő - idő);
            if (MaximálisSebesség < sebesség)
            {
                MaximálisSebesség = sebesség;
            }

            // Érkezési pont "kiszámítása"
            ÉrkezésiPont = v;

            // előző vektor
            ElőzőVektor = v;
            UtolsóIdő = idő;
        }
    }
}
