﻿using System;
using System.Linq;
using System.IO;

namespace CCars
{
    class Térkép
    {
        /// <summary>
        /// Új <see cref="T:CCars.Térkép"/> példány létrehozása. (csak ebből az osztályból elérhető)
        /// </summary>
        /// <param name="adatok">Adatok.</param>
        private Térkép(int[][] adatok)
        {
            this.adatok = adatok;
        }

        /// <summary>
        /// Térkép szélességének lekérdezése
        /// </summary>
        /// <value>The szélesség.</value>
        public int Szélesség => adatok[0].Length;

        /// <summary>
        /// Térkép magasságának lekérdezése
        /// </summary>
        public int Magasság => adatok.Length;

        /// <summary>
        /// Térkép adatai
        /// </summary>
        private int[][] adatok;

        /// <summary>
        /// Rajta van a pont a térképen?
        /// </summary>
        /// <returns><c>true</c>, ha rajta van, <c>false</c> egyébként.</returns>
        /// <param name="x">X koordináta</param>
        /// <param name="y">Y koordináta</param>
        public bool VanÚt(int x, int y)
        {
            if (x < 0 || x >= Szélesség)
            {
                return false;
            }

            if (y < 0 || y >= Magasság)
            {
                return false;
            }

            return adatok[y][x] != 0;
        }

        /// <summary>
        /// Sebességkorlátozás adott ponton
        /// </summary>
        /// <returns>A sebességkorlát</returns>
        /// <param name="x">X koordináta</param>
        /// <param name="y">Y koordináta</param>
        public int SebességKorlát(int x, int y)
        {
            if (!VanÚt(x, y))
            {
                throw new ArgumentException("Nincs út");
            }
            return adatok[y][x];
        }

        /// <summary>
        /// Térkép beolvasása fájlból
        /// </summary>
        /// <returns>Beolvasott térkép</returns>
        /// <param name="fájl">Fájl neve a beolvasáshoz</param>
        public static Térkép Beolvas(string fájl) => new Térkép(
            File.ReadLines(fájl).Select(sor => sor.Split().Select(int.Parse).ToArray()).Reverse().ToArray()
        );

        /// <summary>
        /// Üres térkép létrehozása
        /// </summary>
        /// <returns>Üres térkép</returns>
        /// <param name="szélesség">Szélesség</param>
        /// <param name="magasság">Magasság</param>
        public static Térkép Üres(int szélesség, int magasság) => new Térkép ( 
                       Enumerable.Repeat(new int[magasság], szélesség).ToArray()
        );
    }
}
