﻿using System;

namespace CCars
{
    /// <summary>
    /// Egy pont a térképen
    /// </summary>
    struct Vektor
    {
        /// <summary>
        /// X koordináta
        /// </summary>
        public float X;

        /// <summary>
        /// Y koordináta
        /// </summary>
        public float Y;

        /// <summary>
        /// Új <see cref="T:CCars.Vektor"/> példány
        /// </summary>
        /// <param name="x">X koordináta</param>
        /// <param name="y">Y koordináta</param>
        public Vektor(float x, float y)
        {
            X = x;
            Y = y;
        }

        /// <summary>
        /// Vektor hossza
        /// </summary>
        /// <value>Vektor hossza</value>
        public float Hossz => (float) Math.Sqrt(X * X + Y * Y);

        /*
         * Műveletek a vektorokkal
         */
        public static Vektor operator +(Vektor u, Vektor v) => new Vektor(u.X + v.X, u.Y + v.Y);
        public static Vektor operator -(Vektor u, Vektor v) => new Vektor(u.X - v.X, u.Y - v.Y);
        public static Vektor operator +(Vektor u) => u;
        public static Vektor operator -(Vektor u) => new Vektor(-u.X, -u.Y);
        public static Vektor operator *(Vektor v, float l) => new Vektor(v.X *l, v.Y * l);
        public static Vektor operator *(float l, Vektor v) => v * l;
        public static Vektor operator /(Vektor v, float l) => new Vektor(v.X / l, v.Y / l);

        /// <summary>
        /// Két helyvektor távolsága
        /// </summary>
        /// <returns>The távolság.</returns>
        /// <param name="u">U vektor</param>
        /// <param name="v">V vektor</param>
        public static float Távolság(Vektor u, Vektor v) => (u - v).Hossz;

        /// <summary>
        /// Vektor szöveg reprezentációja
        /// </summary>
        /// <returns>Vektor szöveg reprezentációja</returns>
        public override string ToString() => $"({X}, {Y})";
    }
}
